#!/usr/bin/python
# vm.py
#
# Generate Xen config file from world file and names
# (filenames and hostname) from vmName.

import sys, os
import ConfigParser
import messages

class VirtualMachine:

	# __init__#{{{
	#
	# Attributes:
	# cn		class #
	# ln		lab #
	# gn		group #
	# vmid		numeric vm id, from world
	# br		name of bridge from world 
	# cpo		ConfigParser object from World class
	# pre		lab directory
	#
	def __init__ (self, cn, ln, gn, vmid, br, cpo, pre ):

		# Data for this VM
		self.classNo = cn
		self.labNo = ln
		self.groupNo = gn
		self.vmId = vmid
		self.vmName = "%s-%s-%s-%s" % \
				(self.classNo, self.labNo, self.groupNo, self.vmId)

		# Paths
		self.prefix = pre		# lab directory
		self.xenFile = self.prefix + "/" + self.vmName #+ ".conf"

		self.verboseFlag = True

		# Xen attributes
		self.name = ""			
		self.memory = ""
		self.disk = ""			
		self.useDHCP = False	# boolean: True = DHCP or False = static
		self.ip = ""
		self.netmask = ""
		self.mac = self.generateInterfaceMACAddress()
		self.bridge = br

		# Get configuration file (world) file
		self.setAttributesFromConfigObject(cpo)
		
		# Generate Xen config file, if none exists in group directory
		if os.path.exists(self.xenFile):
			messages.info("Found config file.")
		else:
			messages.info("Config file not found.")
			messages.info("Generating config file %s" % (self.xenFile))
			self.generateXenConfigFile()


#}}}
# setAttributesFromConfigObject {{{1
	#  Gets and sets configuration from ConfigParser object
	#
	# Arguments:
	# cpo		ConfigParser object passed from XenWorld object
	#
	# implement name and conf later
	def setAttributesFromConfigObject (self, cpo):
		vmSection = "Node " + self.vmId

		try:
			self.memory = cpo.get(vmSection, "memory")
			self.disk = cpo.get(vmSection, "disk")
			self.netmask = cpo.get(vmSection, "netmask")
			self.ip = cpo.get(vmSection, "ip")

		except ConfigParser.NoSectionError:
			messages.error("No section \'%s\' in configuration" % vmSection)


	# generateXenConfigFile {{{1
	#  Creates a Xen guest configuration file at self.xenFile
	#
	# Arguments:
	# none
	#
	def generateXenConfigFile (self):
		if not os.path.isfile(self.xenFile):
			CF = open(self.xenFile, "w")
			messages.info("Writing config file: %s" % (self.xenFile))

			# Basic required VM information
			CF.write("name = '%s'\n" % self.vmName)
			CF.write("kernel = '/boot/vmlinuz-%s'\n" % (os.uname()[2]))
			CF.write("ramdisk = '/boot/initrd.img-%s'\n" % (os.uname()[2]))
			CF.write("memory = %s\n" % self.memory)

			# Disk information
			# root will default to xvda1
			# test this on deckard - xvda vs. sda
			CF.write("root = '/dev/sda1 ro'\n")
			# CF.write("disk = [ \'file:%s,hda,w\' ]\n" % self.disk)
			# Open disk= section
			CF.write("disk \t= [\n")
			self.diskList = self.disk.split(",")
			for i in range(len(self.diskList)):
				CF.write("\t\t\'file:%s,sda%s,w\'" \
						% (self.diskList[i].strip(), str(i+1)))
				if i < len(self.diskList)-1:
					CF.write(",\n")

			CF.write("\n\t  ]\n")

			# Network information
			CF.write("vif = [ \'ip=%s,mac=%s,bridge=%s\' ]\n" % \
				(self.ip, self.mac, self.bridge))
                        CF.write("netmask = \"%s\"\n" % self.netmask)

			# Only use if you want xm console access, and not vfb
			# CF.write("extra = 'console=hvc xencons=tty'\n")
			# CF.write("nographic=1\n")
			CF.write("vfb = ['type=vnc, vncunused=1']\n")
			CF.write("vcpus=1\n")
			CF.write("on_reboot=\'restart\'\n")
			CF.write("on_crash=\'restart\'\n")

			CF.close()
		else:
			messages.info("Xen config file for %s already exists." \
				% (self.vmName))

	# generateInterfaceMACAddress
	#  Generates a unique MAC address for this VM's network interface
	#  XENLABSPREFIX = classNo+labNo (unique)
	# 	ACTUALLY I DON'T THINK THIS IS UNIQUE. RECONSIDER
	# Arguments:
	# none
	#
	def generateInterfaceMACAddress (self):
		XENPREFIX="00:16:3e"
		# watch out for string vs. int weirdness
		# will have to standardize all this at some point
		XENLABSPREFIX = self.classNo + self.labNo
		macAddr = "%s:%02x:%02x:%02x" % \
				(XENPREFIX, int(XENLABSPREFIX), int(self.groupNo), \
				int(self.vmId)+1)

				#if self.verboseFlag:
	#		messages.debug("Generating MAC address for VM %s: %s" 
	#				% (self.vmId, macAddr))

		return macAddr



	# printVMInfo {{{1
	#  Prints the VM info in a nice easy-to-read way.
	#
	# Arguments:
	# none
	#
	def printVMInfo (self):
		print "		VM Name: %s" % (self.vmName)
		print "		  VM ID: %s" % (self.vmId)
		print "	  Lab No: %s" % (self.labNo)
		print "Xen Config File: %s" % (self.xenFile)
		print "	 -- Xen Attributes --"
		print "	   Hostname: %s" % (self.name)
		print "		    RAM: %s MB" % (self.memory)
		print "		   Disk: %s" % (self.disk)
		print "	 IP Address: %s" % (self.ip)
		print "		Netmask: %s" % (self.netmask)
		print "	MAC Address: %s" % (self.mac)
		print "		 Bridge: %s" % (self.bridge)


	# startVM {{{1
	#  Call xm command to create/start VM
	#
	# Arguments:
	# none
	#
	def startVM (self):
		command = "sudo /usr/sbin/xm create " + self.xenFile

		if self.verboseFlag:
			messages.debug("Executing command on VM %s: %s" 
					% (self.vmId, command))

		result = os.system(command)
		if result == 0:
			messages.info("Started virtual machine using file: " + self.xenFile)
		else:
			messages.error("Problem starting virtual machine with config: " + \
					self.xenFile)



	# stopVM {{{1
	#  Call xm command to shutdown this VM cleanly
	#
	# Arguments:
	# none
	#
	def stopVM (self):
		command = "sudo /usr/sbin/xm shutdown " + self.vmName

		result = os.system(command)
		if result == 0:
			messages.info("Shutdown virtual machine: " + self.vmName)
		else:
			messages.error("Problem shutting down virtual machine: " + \
				self.vmName)



	# rebootVM
	#  Call xm command to reboot this VM
	#
	# Arguments:
	#  none
	#
	def rebootVM (self):
		command = "sudo /usr/sbin/xm reboot " + self.vmName

		result = os.system(command)
		if result == 0:
			messages.info("Rebooting %s" % (self.vmName))
		else:
			messages.error("Problem rebooting virtual machine: " + \
					self.vmName)



	# destroyVM {{{1
	#  Call xm command to destroy this VM
	#
	# Arguments:
	# none
	#
	def destroyVM (self):
		command = "sudo /usr/sbin/xm destroy " + self.vmName

		if self.verboseFlag:
			messages.debug("Executing command on VM %s: %s" 
					% (self.vmId, command))

		result = os.system(command)
		if result == 0:
			messages.info("Destroyed virtual machine: " + self.vmName)
		else:
			messages.error("Problem destroying virtual machine: " + \
				self.vmName)

#}}}
# main() #{{{
# for debugging purposes only
def main():
	pass

if __name__ == "__main__":
	main()
#}}}

# vim:fdm=marker
